@@ -23,6 +23,7 @@ from osc_lib import utils
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import sdk_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -84,6 +85,15 @@ def _get_common_parse_arguments(parser, is_create=True):
         help=_("DNS server for this subnet "
                "(repeat option to set multiple DNS servers)")
     )
+
+    if not is_create:
+        parser.add_argument(
+            '--no-dns-nameservers',
+            action='store_true',
+            help=_("Clear existing information of DNS Nameservers. "
+                   "Specify both --dns-nameserver and --no-dns-nameserver "
+                   "to overwrite the current DNS Nameserver information.")
+        )
     parser.add_argument(
         '--host-route',
         metavar='destination=<subnet>,gateway=<ip-address>',
@@ -117,11 +127,12 @@ def _get_common_parse_arguments(parser, is_create=True):
 
 
 def _get_columns(item):
-    columns = list(item.keys())
-    if 'tenant_id' in columns:
-        columns.remove('tenant_id')
-        columns.append('project_id')
-    return tuple(sorted(columns))
+    column_map = {
+        'is_dhcp_enabled': 'enable_dhcp',
+        'subnet_pool_id': 'subnetpool_id',
+        'tenant_id': 'project_id',
+    }
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
 
 
 def convert_entries_to_nexthop(entries):
@@ -217,6 +228,8 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
     return attrs
 
 
+# TODO(abhiraut): Use the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class CreateSubnet(command.ShowOne):
     _description = _("Create a subnet")
 
@@ -323,9 +336,9 @@ class CreateSubnet(command.ShowOne):
         client = self.app.client_manager.network
         attrs = _get_attrs(self.app.client_manager, parsed_args)
         obj = client.create_subnet(**attrs)
-        columns = _get_columns(obj)
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
-        return (columns, data)
+        return (display_columns, data)
 
 
 class DeleteSubnet(command.Command):
@@ -352,8 +365,8 @@ class DeleteSubnet(command.Command):
             except Exception as e:
                 result += 1
                 LOG.error(_("Failed to delete subnet with "
-                          "name or ID '%(subnet)s': %(e)s")
-                          % {'subnet': subnet, 'e': e})
+                          "name or ID '%(subnet)s': %(e)s"),
+                          {'subnet': subnet, 'e': e})
 
         if result > 0:
             total = len(parsed_args.subnet)
@@ -362,6 +375,8 @@ class DeleteSubnet(command.Command):
             raise exceptions.CommandError(msg)
 
 
+# TODO(abhiraut): Use only the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class ListSubnet(command.Lister):
     _description = _("List subnets")
 
@@ -443,8 +458,10 @@ class ListSubnet(command.Lister):
             filters['ip_version'] = parsed_args.ip_version
         if parsed_args.dhcp:
             filters['enable_dhcp'] = True
+            filters['is_dhcp_enabled'] = True
         elif parsed_args.no_dhcp:
             filters['enable_dhcp'] = False
+            filters['is_dhcp_enabled'] = False
         if parsed_args.service_types:
             filters['service_types'] = parsed_args.service_types
         if parsed_args.project:
@@ -454,6 +471,7 @@ class ListSubnet(command.Lister):
                 parsed_args.project_domain,
             ).id
             filters['tenant_id'] = project_id
+            filters['project_id'] = project_id
         if parsed_args.network:
             network_id = network_client.find_network(parsed_args.network,
                                                      ignore_missing=False).id
@@ -472,7 +490,7 @@ class ListSubnet(command.Lister):
             headers += ('Project', 'DHCP', 'Name Servers',
                         'Allocation Pools', 'Host Routes', 'IP Version',
                         'Gateway', 'Service Types')
-            columns += ('tenant_id', 'enable_dhcp', 'dns_nameservers',
+            columns += ('project_id', 'is_dhcp_enabled', 'dns_nameservers',
                         'allocation_pools', 'host_routes', 'ip_version',
                         'gateway_ip', 'service_types')
 
@@ -483,6 +501,8 @@ class ListSubnet(command.Lister):
                 ) for s in data))
 
 
+# TODO(abhiraut): Use the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class SetSubnet(command.Command):
     _description = _("Set subnet properties")
 
@@ -532,7 +552,10 @@ class SetSubnet(command.Command):
         attrs = _get_attrs(self.app.client_manager, parsed_args,
                            is_create=False)
         if 'dns_nameservers' in attrs:
-            attrs['dns_nameservers'] += obj.dns_nameservers
+            if not parsed_args.no_dns_nameservers:
+                attrs['dns_nameservers'] += obj.dns_nameservers
+        elif parsed_args.no_dns_nameservers:
+            attrs['dns_nameservers'] = []
         if 'host_routes' in attrs:
             if not parsed_args.no_host_route:
                 attrs['host_routes'] += obj.host_routes
@@ -564,9 +587,9 @@ class ShowSubnet(command.ShowOne):
     def take_action(self, parsed_args):
         obj = self.app.client_manager.network.find_subnet(parsed_args.subnet,
                                                           ignore_missing=False)
-        columns = _get_columns(obj)
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
-        return (columns, data)
+        return (display_columns, data)
 
 
 class UnsetSubnet(command.Command):
