@@ -19,6 +19,7 @@ from mock import call
 
 from osc_lib import exceptions
 from osc_lib import utils as common_utils
+from oslo_utils import timeutils
 
 from openstackclient.compute.v2 import server
 from openstackclient.tests.unit.compute.v2 import fakes as compute_fakes
@@ -36,10 +37,6 @@ class TestServer(compute_fakes.TestComputev2):
         self.servers_mock = self.app.client_manager.compute.servers
         self.servers_mock.reset_mock()
 
-        # Get a shortcut to the compute client ImageManager Mock
-        self.cimages_mock = self.app.client_manager.compute.images
-        self.cimages_mock.reset_mock()
-
         # Get a shortcut to the compute client FlavorManager Mock
         self.flavors_mock = self.app.client_manager.compute.flavors
         self.flavors_mock.reset_mock()
@@ -259,7 +256,7 @@ class TestServerCreate(TestServer):
         self.servers_mock.create.return_value = self.new_server
 
         self.image = image_fakes.FakeImage.create_one_image()
-        self.cimages_mock.get.return_value = self.image
+        self.images_mock.get.return_value = self.image
 
         self.flavor = compute_fakes.FakeFlavor.create_one_flavor()
         self.flavors_mock.get.return_value = self.flavor
@@ -312,7 +309,7 @@ class TestServerCreate(TestServer):
             userdata=None,
             key_name=None,
             availability_zone=None,
-            block_device_mapping={},
+            block_device_mapping_v2=[],
             nics=[],
             scheduler_hints={},
             config_drive=None,
@@ -367,7 +364,7 @@ class TestServerCreate(TestServer):
             userdata=None,
             key_name='keyname',
             availability_zone=None,
-            block_device_mapping={},
+            block_device_mapping_v2=[],
             nics=[],
             scheduler_hints={'a': ['b', 'c']},
             config_drive=None,
@@ -447,7 +444,7 @@ class TestServerCreate(TestServer):
             userdata=None,
             key_name=None,
             availability_zone=None,
-            block_device_mapping={},
+            block_device_mapping_v2=[],
             nics=[{'net-id': 'net1_uuid',
                    'v4-fixed-ip': '',
                    'v6-fixed-ip': '',
@@ -470,6 +467,152 @@ class TestServerCreate(TestServer):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist(), data)
 
+    def test_server_create_with_auto_network(self):
+        arglist = [
+            '--image', 'image1',
+            '--flavor', 'flavor1',
+            '--nic', 'auto',
+            self.new_server.name,
+        ]
+        verifylist = [
+            ('image', 'image1'),
+            ('flavor', 'flavor1'),
+            ('nic', ['auto']),
+            ('config_drive', False),
+            ('server_name', self.new_server.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = dict(
+            meta=None,
+            files={},
+            reservation_id=None,
+            min_count=1,
+            max_count=1,
+            security_groups=[],
+            userdata=None,
+            key_name=None,
+            availability_zone=None,
+            block_device_mapping_v2=[],
+            nics='auto',
+            scheduler_hints={},
+            config_drive=None,
+        )
+        # ServerManager.create(name, image, flavor, **kwargs)
+        self.servers_mock.create.assert_called_with(
+            self.new_server.name,
+            self.image,
+            self.flavor,
+            **kwargs
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist(), data)
+
+    def test_server_create_with_none_network(self):
+        arglist = [
+            '--image', 'image1',
+            '--flavor', 'flavor1',
+            '--nic', 'none',
+            self.new_server.name,
+        ]
+        verifylist = [
+            ('image', 'image1'),
+            ('flavor', 'flavor1'),
+            ('nic', ['none']),
+            ('config_drive', False),
+            ('server_name', self.new_server.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = dict(
+            meta=None,
+            files={},
+            reservation_id=None,
+            min_count=1,
+            max_count=1,
+            security_groups=[],
+            userdata=None,
+            key_name=None,
+            availability_zone=None,
+            block_device_mapping_v2=[],
+            nics='none',
+            scheduler_hints={},
+            config_drive=None,
+        )
+        # ServerManager.create(name, image, flavor, **kwargs)
+        self.servers_mock.create.assert_called_with(
+            self.new_server.name,
+            self.image,
+            self.flavor,
+            **kwargs
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist(), data)
+
+    def test_server_create_with_conflict_network_options(self):
+        arglist = [
+            '--image', 'image1',
+            '--flavor', 'flavor1',
+            '--nic', 'none',
+            '--nic', 'auto',
+            '--nic', 'port-id=port1',
+            self.new_server.name,
+        ]
+        verifylist = [
+            ('image', 'image1'),
+            ('flavor', 'flavor1'),
+            ('nic', ['none', 'auto', 'port-id=port1']),
+            ('config_drive', False),
+            ('server_name', self.new_server.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        get_endpoints = mock.Mock()
+        get_endpoints.return_value = {'network': []}
+        self.app.client_manager.auth_ref = mock.Mock()
+        self.app.client_manager.auth_ref.service_catalog = mock.Mock()
+        self.app.client_manager.auth_ref.service_catalog.get_endpoints = (
+            get_endpoints)
+
+        find_port = mock.Mock()
+        network_client = self.app.client_manager.network
+        network_client.find_port = find_port
+        port_resource = mock.Mock()
+        port_resource.id = 'port1_uuid'
+        find_port.return_value = port_resource
+
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action, parsed_args)
+        self.assertNotCalled(self.servers_mock.create)
+
+    def test_server_create_with_invalid_network_options(self):
+        arglist = [
+            '--image', 'image1',
+            '--flavor', 'flavor1',
+            '--nic', 'abcdefgh',
+            self.new_server.name,
+        ]
+        verifylist = [
+            ('image', 'image1'),
+            ('flavor', 'flavor1'),
+            ('nic', ['abcdefgh']),
+            ('config_drive', False),
+            ('server_name', self.new_server.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action, parsed_args)
+        self.assertNotCalled(self.servers_mock.create)
+
     @mock.patch.object(common_utils, 'wait_for_status', return_value=True)
     def test_server_create_with_wait_ok(self, mock_wait_for_status):
         arglist = [
@@ -504,7 +647,7 @@ class TestServerCreate(TestServer):
             userdata=None,
             key_name=None,
             availability_zone=None,
-            block_device_mapping={},
+            block_device_mapping_v2=[],
             nics=[],
             scheduler_hints={},
             config_drive=None,
@@ -554,7 +697,7 @@ class TestServerCreate(TestServer):
             userdata=None,
             key_name=None,
             availability_zone=None,
-            block_device_mapping={},
+            block_device_mapping_v2=[],
             nics=[],
             scheduler_hints={},
             config_drive=None,
@@ -609,7 +752,7 @@ class TestServerCreate(TestServer):
             userdata=mock_file,
             key_name=None,
             availability_zone=None,
-            block_device_mapping={},
+            block_device_mapping_v2=[],
             nics=[],
             scheduler_hints={},
             config_drive=None,
@@ -660,9 +803,14 @@ class TestServerCreate(TestServer):
             userdata=None,
             key_name=None,
             availability_zone=None,
-            block_device_mapping={
-                'vda': real_volume_mapping
-            },
+            block_device_mapping_v2=[{
+                'device_name': 'vda',
+                'uuid': real_volume_mapping.split(':', 1)[0],
+                'destination_type': 'volume',
+                'source_type': 'volume',
+                'delete_on_termination': '0',
+                'volume_size': ''
+            }],
             nics=[],
             scheduler_hints={},
             config_drive=None,
@@ -830,6 +978,8 @@ class TestServerList(TestServer):
             'tenant_id': None,
             'all_tenants': False,
             'user_id': None,
+            'deleted': False,
+            'changes_since': None,
         }
 
         # Default params of the core function of the command in the case of no
@@ -859,7 +1009,7 @@ class TestServerList(TestServer):
         self.servers_mock.list.return_value = self.servers
 
         self.image = image_fakes.FakeImage.create_one_image()
-        self.cimages_mock.get.return_value = self.image
+        self.images_mock.get.return_value = self.image
 
         self.flavor = compute_fakes.FakeFlavor.create_one_flavor()
         self.flavors_mock.get.return_value = self.flavor
@@ -906,6 +1056,7 @@ class TestServerList(TestServer):
         verifylist = [
             ('all_projects', False),
             ('long', False),
+            ('deleted', False),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -943,7 +1094,7 @@ class TestServerList(TestServer):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.cimages_mock.get.assert_any_call(self.image.id)
+        self.images_mock.get.assert_any_call(self.image.id)
 
         self.search_opts['image'] = self.image.id
         self.servers_mock.list.assert_called_with(**self.kwargs)
@@ -971,6 +1122,48 @@ class TestServerList(TestServer):
         self.assertEqual(self.columns, columns)
         self.assertEqual(tuple(self.data), tuple(data))
 
+    def test_server_list_with_changes_since(self):
+
+        arglist = [
+            '--changes-since', '2016-03-04T06:27:59Z',
+            '--deleted'
+        ]
+        verifylist = [
+            ('changes_since', '2016-03-04T06:27:59Z'),
+            ('deleted', True),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.search_opts['changes_since'] = '2016-03-04T06:27:59Z'
+        self.search_opts['deleted'] = True
+        self.servers_mock.list.assert_called_with(**self.kwargs)
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(tuple(self.data), tuple(data))
+
+    @mock.patch.object(timeutils, 'parse_isotime', side_effect=ValueError)
+    def test_server_list_with_invalid_changes_since(self, mock_parse_isotime):
+
+        arglist = [
+            '--changes-since', 'Invalid time value',
+        ]
+        verifylist = [
+            ('changes_since', 'Invalid time value'),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        try:
+            self.cmd.take_action(parsed_args)
+            self.fail('CommandError should be raised.')
+        except exceptions.CommandError as e:
+            self.assertEqual('Invalid changes-since value: Invalid time '
+                             'value', str(e))
+        mock_parse_isotime.assert_called_once_with(
+            'Invalid time value'
+        )
+
 
 class TestServerLock(TestServer):
 
@@ -1019,7 +1212,7 @@ class TestServerRebuild(TestServer):
 
         # Return value for utils.find_resource for image
         self.image = image_fakes.FakeImage.create_one_image()
-        self.cimages_mock.get.return_value = self.image
+        self.images_mock.get.return_value = self.image
 
         # Fake the rebuilt new server.
         new_server = compute_fakes.FakeServer.create_one_server()
@@ -1059,7 +1252,7 @@ class TestServerRebuild(TestServer):
         self.cmd.take_action(parsed_args)
 
         self.servers_mock.get.assert_called_with(self.server.id)
-        self.cimages_mock.get.assert_called_with(self.image.id)
+        self.images_mock.get.assert_called_with(self.image.id)
         self.server.rebuild.assert_called_with(self.image, None)
 
     def test_rebuild_with_current_image_and_password(self):
@@ -1078,7 +1271,7 @@ class TestServerRebuild(TestServer):
         self.cmd.take_action(parsed_args)
 
         self.servers_mock.get.assert_called_with(self.server.id)
-        self.cimages_mock.get.assert_called_with(self.image.id)
+        self.images_mock.get.assert_called_with(self.image.id)
         self.server.rebuild.assert_called_with(self.image, password)
 
     @mock.patch.object(common_utils, 'wait_for_status', return_value=True)
@@ -1106,7 +1299,7 @@ class TestServerRebuild(TestServer):
         )
 
         self.servers_mock.get.assert_called_with(self.server.id)
-        self.cimages_mock.get.assert_called_with(self.image.id)
+        self.images_mock.get.assert_called_with(self.image.id)
         self.server.rebuild.assert_called_with(self.image, None)
 
     @mock.patch.object(common_utils, 'wait_for_status', return_value=False)
@@ -1130,7 +1323,7 @@ class TestServerRebuild(TestServer):
         )
 
         self.servers_mock.get.assert_called_with(self.server.id)
-        self.cimages_mock.get.assert_called_with(self.image.id)
+        self.images_mock.get.assert_called_with(self.image.id)
         self.server.rebuild.assert_called_with(self.image, None)
 
 
@@ -1628,7 +1821,7 @@ class TestServerShow(TestServer):
 
         # This is the return value for utils.find_resource()
         self.servers_mock.get.return_value = self.server
-        self.cimages_mock.get.return_value = self.image
+        self.images_mock.get.return_value = self.image
         self.flavors_mock.get.return_value = self.flavor
 
         # Get the command object to test
@@ -1986,6 +2179,7 @@ class TestServerGeneral(TestServer):
         # Call _prep_server_detail().
         server_detail = server._prep_server_detail(
             self.app.client_manager.compute,
+            self.app.client_manager.image,
             _server
         )
         # 'networks' is used to create _server. Remove it.
