@@ -20,6 +20,7 @@ from osc_lib import utils
 
 from openstackclient.network.v2 import port
 from openstackclient.tests.unit.compute.v2 import fakes as compute_fakes
+from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes
 from openstackclient.tests.unit.network.v2 import fakes as network_fakes
 from openstackclient.tests.unit import utils as tests_utils
 
@@ -31,6 +32,8 @@ class TestPort(network_fakes.TestNetworkV2):
 
         # Get a shortcut to the network client
         self.network = self.app.client_manager.network
+        # Get a shortcut to the ProjectManager Mock
+        self.projects_mock = self.app.client_manager.identity.projects
 
     def _get_common_cols_data(self, fake_port):
         columns = (
@@ -54,7 +57,7 @@ class TestPort(network_fakes.TestNetworkV2):
             'network_id',
             'port_security_enabled',
             'project_id',
-            'security_groups',
+            'security_group_ids',
             'status',
         )
 
@@ -79,7 +82,7 @@ class TestPort(network_fakes.TestNetworkV2):
             fake_port.network_id,
             fake_port.port_security_enabled,
             fake_port.project_id,
-            utils.format_list(fake_port.security_groups),
+            utils.format_list(fake_port.security_group_ids),
             fake_port.status,
         )
 
@@ -216,7 +219,7 @@ class TestCreatePort(TestPort):
             'test-port',
         ]
         verifylist = [
-            ('network', self._port.network_id,),
+            ('network', self._port.network_id),
             ('enable', True),
             ('binding_profile', {'parent_name': 'fake_parent', 'tag': 42}),
             ('name', 'test-port'),
@@ -248,7 +251,7 @@ class TestCreatePort(TestPort):
         verifylist = [
             ('network', self._port.network_id,),
             ('enable', True),
-            ('security_groups', [secgroup.id]),
+            ('security_group', [secgroup.id]),
             ('name', 'test-port'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -258,7 +261,7 @@ class TestCreatePort(TestPort):
         self.network.create_port.assert_called_once_with(**{
             'admin_state_up': True,
             'network_id': self._port.network_id,
-            'security_groups': [secgroup.id],
+            'security_group_ids': [secgroup.id],
             'name': 'test-port',
         })
 
@@ -306,7 +309,7 @@ class TestCreatePort(TestPort):
         verifylist = [
             ('network', self._port.network_id,),
             ('enable', True),
-            ('security_groups', [sg_1.id, sg_2.id]),
+            ('security_group', [sg_1.id, sg_2.id]),
             ('name', 'test-port'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -316,7 +319,7 @@ class TestCreatePort(TestPort):
         self.network.create_port.assert_called_once_with(**{
             'admin_state_up': True,
             'network_id': self._port.network_id,
-            'security_groups': [sg_1.id, sg_2.id],
+            'security_group_ids': [sg_1.id, sg_2.id],
             'name': 'test-port',
         })
 
@@ -343,7 +346,75 @@ class TestCreatePort(TestPort):
         self.network.create_port.assert_called_once_with(**{
             'admin_state_up': True,
             'network_id': self._port.network_id,
-            'security_groups': [],
+            'security_group_ids': [],
+            'name': 'test-port',
+        })
+
+        ref_columns, ref_data = self._get_common_cols_data(self._port)
+        self.assertEqual(ref_columns, columns)
+        self.assertEqual(ref_data, data)
+
+    def test_create_port_with_allowed_address_pair_ipaddr(self):
+        pairs = [{'ip_address': '192.168.1.123'},
+                 {'ip_address': '192.168.1.45'}]
+        arglist = [
+            '--network', self._port.network_id,
+            '--allowed-address', 'ip-address=192.168.1.123',
+            '--allowed-address', 'ip-address=192.168.1.45',
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id),
+            ('enable', True),
+            ('allowed_address_pairs', [{'ip-address': '192.168.1.123'},
+                                       {'ip-address': '192.168.1.45'}]),
+            ('name', 'test-port'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'allowed_address_pairs': pairs,
+            'name': 'test-port',
+        })
+
+        ref_columns, ref_data = self._get_common_cols_data(self._port)
+        self.assertEqual(ref_columns, columns)
+        self.assertEqual(ref_data, data)
+
+    def test_create_port_with_allowed_address_pair(self):
+        pairs = [{'ip_address': '192.168.1.123',
+                  'mac_address': 'aa:aa:aa:aa:aa:aa'},
+                 {'ip_address': '192.168.1.45',
+                  'mac_address': 'aa:aa:aa:aa:aa:b1'}]
+        arglist = [
+            '--network', self._port.network_id,
+            '--allowed-address',
+            'ip-address=192.168.1.123,mac-address=aa:aa:aa:aa:aa:aa',
+            '--allowed-address',
+            'ip-address=192.168.1.45,mac-address=aa:aa:aa:aa:aa:b1',
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id),
+            ('enable', True),
+            ('allowed_address_pairs', [{'ip-address': '192.168.1.123',
+                                        'mac-address': 'aa:aa:aa:aa:aa:aa'},
+                                       {'ip-address': '192.168.1.45',
+                                        'mac-address': 'aa:aa:aa:aa:aa:b1'}]),
+            ('name', 'test-port'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'allowed_address_pairs': pairs,
             'name': 'test-port',
         })
 
@@ -519,7 +590,7 @@ class TestListPort(TestPort):
             prt.mac_address,
             utils.format_list_of_dicts(prt.fixed_ips),
             prt.status,
-            utils.format_list(prt.security_groups),
+            utils.format_list(prt.security_group_ids),
             prt.device_owner,
         ))
 
@@ -656,6 +727,92 @@ class TestListPort(TestPort):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
+    def test_port_list_fixed_ip_opt_ip_address(self):
+        ip_address = self._ports[0].fixed_ips[0]['ip_address']
+        arglist = [
+            '--fixed-ip', "ip-address=%s" % ip_address,
+        ]
+        verifylist = [
+            ('fixed_ip', [{'ip-address': ip_address}])
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.ports.assert_called_once_with(**{
+            'fixed_ips': ['ip_address=%s' % ip_address]})
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_port_list_fixed_ip_opt_subnet_id(self):
+        subnet_id = self._ports[0].fixed_ips[0]['subnet_id']
+        arglist = [
+            '--fixed-ip', "subnet=%s" % subnet_id,
+        ]
+        verifylist = [
+            ('fixed_ip', [{'subnet': subnet_id}])
+        ]
+
+        self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet(
+            {'id': subnet_id})
+        self.network.find_subnet = mock.Mock(return_value=self.fake_subnet)
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.ports.assert_called_once_with(**{
+            'fixed_ips': ['subnet_id=%s' % subnet_id]})
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_port_list_fixed_ip_opts(self):
+        subnet_id = self._ports[0].fixed_ips[0]['subnet_id']
+        ip_address = self._ports[0].fixed_ips[0]['ip_address']
+        arglist = [
+            '--fixed-ip', "subnet=%s,ip-address=%s" % (subnet_id,
+                                                       ip_address)
+        ]
+        verifylist = [
+            ('fixed_ip', [{'subnet': subnet_id,
+                          'ip-address': ip_address}])
+        ]
+
+        self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet(
+            {'id': subnet_id})
+        self.network.find_subnet = mock.Mock(return_value=self.fake_subnet)
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.ports.assert_called_once_with(**{
+            'fixed_ips': ['subnet_id=%s' % subnet_id,
+                          'ip_address=%s' % ip_address]})
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_port_list_fixed_ips(self):
+        subnet_id = self._ports[0].fixed_ips[0]['subnet_id']
+        ip_address = self._ports[0].fixed_ips[0]['ip_address']
+        arglist = [
+            '--fixed-ip', "subnet=%s" % subnet_id,
+            '--fixed-ip', "ip-address=%s" % ip_address,
+        ]
+        verifylist = [
+            ('fixed_ip', [{'subnet': subnet_id},
+                          {'ip-address': ip_address}])
+        ]
+
+        self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet(
+            {'id': subnet_id})
+        self.network.find_subnet = mock.Mock(return_value=self.fake_subnet)
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.ports.assert_called_once_with(**{
+            'fixed_ips': ['subnet_id=%s' % subnet_id,
+                          'ip_address=%s' % ip_address]})
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
     def test_list_port_with_long(self):
         arglist = [
             '--long',
@@ -673,6 +830,44 @@ class TestListPort(TestPort):
         self.assertEqual(self.columns_long, columns)
         self.assertEqual(self.data_long, list(data))
 
+    def test_port_list_project(self):
+        project = identity_fakes.FakeProject.create_one_project()
+        self.projects_mock.get.return_value = project
+        arglist = [
+            '--project', project.id,
+        ]
+        verifylist = [
+            ('project', project.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        filters = {'tenant_id': project.id, 'project_id': project.id}
+
+        self.network.ports.assert_called_once_with(**filters)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_port_list_project_domain(self):
+        project = identity_fakes.FakeProject.create_one_project()
+        self.projects_mock.get.return_value = project
+        arglist = [
+            '--project', project.id,
+            '--project-domain', project.domain_id,
+        ]
+        verifylist = [
+            ('project', project.id),
+            ('project_domain', project.domain_id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        filters = {'tenant_id': project.id, 'project_id': project.id}
+
+        self.network.ports.assert_called_once_with(**filters)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
 
 class TestSetPort(TestPort):
 
@@ -692,6 +887,7 @@ class TestSetPort(TestPort):
         arglist = [
             '--fixed-ip', 'ip-address=10.0.0.11',
             self._port.name,
+            '--no-fixed-ip',
         ]
         verifylist = [
             ('fixed_ip', [{'ip-address': '10.0.0.11'}]),
@@ -915,7 +1111,7 @@ class TestSetPort(TestPort):
             self._port.name,
         ]
         verifylist = [
-            ('security_groups', [sg.id]),
+            ('security_group', [sg.id]),
             ('port', self._port.name),
         ]
 
@@ -923,7 +1119,7 @@ class TestSetPort(TestPort):
         result = self.cmd.take_action(parsed_args)
 
         attrs = {
-            'security_groups': [sg.id],
+            'security_group_ids': [sg.id],
         }
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
@@ -934,7 +1130,7 @@ class TestSetPort(TestPort):
         sg_3 = network_fakes.FakeSecurityGroup.create_one_security_group()
         self.network.find_security_group = mock.Mock(side_effect=[sg_2, sg_3])
         _testport = network_fakes.FakePort.create_one_port(
-            {'security_groups': [sg_1.id]})
+            {'security_group_ids': [sg_1.id]})
         self.network.find_port = mock.Mock(return_value=_testport)
         arglist = [
             '--security-group', sg_2.id,
@@ -942,13 +1138,13 @@ class TestSetPort(TestPort):
             _testport.name,
         ]
         verifylist = [
-            ('security_groups', [sg_2.id, sg_3.id]),
+            ('security_group', [sg_2.id, sg_3.id]),
             ('port', _testport.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
         attrs = {
-            'security_groups': [sg_1.id, sg_2.id, sg_3.id],
+            'security_group_ids': [sg_2.id, sg_3.id, sg_1.id],
         }
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
@@ -967,7 +1163,7 @@ class TestSetPort(TestPort):
         result = self.cmd.take_action(parsed_args)
 
         attrs = {
-            'security_groups': [],
+            'security_group_ids': [],
         }
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
@@ -976,7 +1172,7 @@ class TestSetPort(TestPort):
         sg1 = network_fakes.FakeSecurityGroup.create_one_security_group()
         sg2 = network_fakes.FakeSecurityGroup.create_one_security_group()
         _testport = network_fakes.FakePort.create_one_port(
-            {'security_groups': [sg1.id]})
+            {'security_group_ids': [sg1.id]})
         self.network.find_port = mock.Mock(return_value=_testport)
         self.network.find_security_group = mock.Mock(return_value=sg2)
         arglist = [
@@ -985,17 +1181,102 @@ class TestSetPort(TestPort):
             _testport.name,
         ]
         verifylist = [
-            ('security_groups', [sg2.id]),
+            ('security_group', [sg2.id]),
             ('no_security_group', True)
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
         attrs = {
-            'security_groups': [sg2.id],
+            'security_group_ids': [sg2.id],
         }
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
 
+    def test_set_allowed_address_pair(self):
+        arglist = [
+            '--allowed-address', 'ip-address=192.168.1.123',
+            self._port.name,
+        ]
+        verifylist = [
+            ('allowed_address_pairs', [{'ip-address': '192.168.1.123'}]),
+            ('port', self._port.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'allowed_address_pairs': [{'ip_address': '192.168.1.123'}],
+        }
+        self.network.update_port.assert_called_once_with(self._port, **attrs)
+        self.assertIsNone(result)
+
+    def test_append_allowed_address_pair(self):
+        _testport = network_fakes.FakePort.create_one_port(
+            {'allowed_address_pairs': [{'ip_address': '192.168.1.123'}]})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        arglist = [
+            '--allowed-address', 'ip-address=192.168.1.45',
+            _testport.name,
+        ]
+        verifylist = [
+            ('allowed_address_pairs', [{'ip-address': '192.168.1.45'}]),
+            ('port', _testport.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'allowed_address_pairs': [{'ip_address': '192.168.1.123'},
+                                      {'ip_address': '192.168.1.45'}],
+        }
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_overwrite_allowed_address_pair(self):
+        _testport = network_fakes.FakePort.create_one_port(
+            {'allowed_address_pairs': [{'ip_address': '192.168.1.123'}]})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        arglist = [
+            '--allowed-address', 'ip-address=192.168.1.45',
+            '--no-allowed-address',
+            _testport.name,
+        ]
+        verifylist = [
+            ('allowed_address_pairs', [{'ip-address': '192.168.1.45'}]),
+            ('no_allowed_address_pair', True),
+            ('port', _testport.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'allowed_address_pairs': [{'ip_address': '192.168.1.45'}],
+        }
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_set_no_allowed_address_pairs(self):
+        arglist = [
+            '--no-allowed-address',
+            self._port.name,
+        ]
+        verifylist = [
+            ('no_allowed_address_pair', True),
+            ('port', self._port.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'allowed_address_pairs': [],
+        }
+        self.network.update_port.assert_called_once_with(self._port, **attrs)
+        self.assertIsNone(result)
+
     def test_port_security_enabled(self):
         arglist = [
             '--enable-port-security',
@@ -1153,7 +1434,7 @@ class TestUnsetPort(TestPort):
         _fake_sg1 = network_fakes.FakeSecurityGroup.create_one_security_group()
         _fake_sg2 = network_fakes.FakeSecurityGroup.create_one_security_group()
         _fake_port = network_fakes.FakePort.create_one_port(
-            {'security_groups': [_fake_sg1.id, _fake_sg2.id]})
+            {'security_group_ids': [_fake_sg1.id, _fake_sg2.id]})
         self.network.find_port = mock.Mock(return_value=_fake_port)
         self.network.find_security_group = mock.Mock(return_value=_fake_sg2)
         arglist = [
@@ -1161,14 +1442,14 @@ class TestUnsetPort(TestPort):
             _fake_port.name,
         ]
         verifylist = [
-            ('security_groups', [_fake_sg2.id]),
+            ('security_group_ids', [_fake_sg2.id]),
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
 
         attrs = {
-            'security_groups': [_fake_sg1.id]
+            'security_group_ids': [_fake_sg1.id]
         }
         self.network.update_port.assert_called_once_with(
             _fake_port, **attrs)
@@ -1178,14 +1459,53 @@ class TestUnsetPort(TestPort):
         _fake_sg1 = network_fakes.FakeSecurityGroup.create_one_security_group()
         _fake_sg2 = network_fakes.FakeSecurityGroup.create_one_security_group()
         _fake_port = network_fakes.FakePort.create_one_port(
-            {'security_groups': [_fake_sg1.id]})
+            {'security_group_ids': [_fake_sg1.id]})
         self.network.find_security_group = mock.Mock(return_value=_fake_sg2)
         arglist = [
             '--security-group', _fake_sg2.id,
             _fake_port.name,
         ]
         verifylist = [
-            ('security_groups', [_fake_sg2.id]),
+            ('security_group_ids', [_fake_sg2.id]),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action,
+                          parsed_args)
+
+    def test_unset_port_allowed_address_pair(self):
+        _fake_port = network_fakes.FakePort.create_one_port(
+            {'allowed_address_pairs': [{'ip_address': '192.168.1.123'}]})
+        self.network.find_port = mock.Mock(return_value=_fake_port)
+        arglist = [
+            '--allowed-address', 'ip-address=192.168.1.123',
+            _fake_port.name,
+        ]
+        verifylist = [
+            ('allowed_address_pairs', [{'ip-address': '192.168.1.123'}]),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'allowed_address_pairs': [],
+        }
+
+        self.network.update_port.assert_called_once_with(_fake_port, **attrs)
+        self.assertIsNone(result)
+
+    def test_unset_port_allowed_address_pair_not_existent(self):
+        _fake_port = network_fakes.FakePort.create_one_port(
+            {'allowed_address_pairs': [{'ip_address': '192.168.1.123'}]})
+        self.network.find_port = mock.Mock(return_value=_fake_port)
+        arglist = [
+            '--allowed-address', 'ip-address=192.168.1.45',
+            _fake_port.name,
+        ]
+        verifylist = [
+            ('allowed_address_pairs', [{'ip-address': '192.168.1.45'}]),
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
