@@ -45,7 +45,7 @@ def _check_size_arg(args):
 
 
 class CreateVolume(command.ShowOne):
-    """Create new volume"""
+    _description = _("Create new volume")
 
     def get_parser(self, prog_name):
         parser = super(CreateVolume, self).get_parser(prog_name)
@@ -114,6 +114,28 @@ class CreateVolume(command.ShowOne):
             help=_('Set a property on this volume '
                    '(repeat option to set multiple properties)'),
         )
+        bootable_group = parser.add_mutually_exclusive_group()
+        bootable_group.add_argument(
+            "--bootable",
+            action="store_true",
+            help=_("Mark volume as bootable")
+        )
+        bootable_group.add_argument(
+            "--non-bootable",
+            action="store_true",
+            help=_("Mark volume as non-bootable (default)")
+        )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode (default)")
+        )
 
         return parser
 
@@ -166,6 +188,22 @@ class CreateVolume(command.ShowOne):
             parsed_args.property,
             image,
         )
+
+        if parsed_args.bootable or parsed_args.non_bootable:
+            try:
+                volume_client.volumes.set_bootable(
+                    volume.id, parsed_args.bootable)
+            except Exception as e:
+                LOG.error(_("Failed to set volume bootable property: %s"), e)
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+
         # Map 'metadata' column to 'properties'
         volume._info.update(
             {
@@ -178,7 +216,7 @@ class CreateVolume(command.ShowOne):
 
 
 class DeleteVolume(command.Command):
-    """Delete volume(s)"""
+    _description = _("Delete volume(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteVolume, self).get_parser(prog_name)
@@ -223,7 +261,7 @@ class DeleteVolume(command.Command):
 
 
 class ListVolume(command.Lister):
-    """List volumes"""
+    _description = _("List volumes")
 
     def get_parser(self, prog_name):
         parser = super(ListVolume, self).get_parser(prog_name)
@@ -344,8 +382,39 @@ class ListVolume(command.Lister):
                 ) for s in data))
 
 
+class MigrateVolume(command.Command):
+    _description = _("Migrate volume to a new host")
+
+    def get_parser(self, prog_name):
+        parser = super(MigrateVolume, self).get_parser(prog_name)
+        parser.add_argument(
+            'volume',
+            metavar="<volume>",
+            help=_("Volume to migrate (name or ID)")
+        )
+        parser.add_argument(
+            '--host',
+            metavar="<host>",
+            required=True,
+            help=_("Destination host (takes the form: host@backend-name#pool)")
+        )
+        parser.add_argument(
+            '--force-host-copy',
+            action="store_true",
+            help=_("Enable generic host-based force-migration, "
+                   "which bypasses driver optimizations")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        volume_client = self.app.client_manager.volume
+        volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
+        volume_client.volumes.migrate_volume(volume.id, parsed_args.host,
+                                             parsed_args.force_host_copy,)
+
+
 class SetVolume(command.Command):
-    """Set volume properties"""
+    _description = _("Set volume properties")
 
     def get_parser(self, prog_name):
         parser = super(SetVolume, self).get_parser(prog_name)
@@ -370,6 +439,15 @@ class SetVolume(command.Command):
             type=int,
             help=_('Extend volume size in GB'),
         )
+        parser.add_argument(
+            "--no-property",
+            dest="no_property",
+            action="store_true",
+            help=_("Remove all properties from <volume> "
+                   "(specify both --no-property and --property to "
+                   "remove the current properties before setting "
+                   "new properties.)"),
+        )
         parser.add_argument(
             '--property',
             metavar='<key=value>',
@@ -388,42 +466,91 @@ class SetVolume(command.Command):
             action="store_true",
             help=_("Mark volume as non-bootable")
         )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode")
+        )
         return parser
 
     def take_action(self, parsed_args):
         volume_client = self.app.client_manager.volume
         volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
 
+        result = 0
         if parsed_args.size:
-            if volume.status != 'available':
-                LOG.error(_("Volume is in %s state, it must be available "
-                            "before size can be extended"), volume.status)
-                return
-            if parsed_args.size <= volume.size:
-                LOG.error(_("New size must be greater than %s GB"),
-                          volume.size)
-                return
-            volume_client.volumes.extend(volume.id, parsed_args.size)
+            try:
+                if volume.status != 'available':
+                    msg = (_("Volume is in %s state, it must be available "
+                           "before size can be extended") % volume.status)
+                    raise exceptions.CommandError(msg)
+                if parsed_args.size <= volume.size:
+                    msg = (_("New size must be greater than %s GB")
+                           % volume.size)
+                    raise exceptions.CommandError(msg)
+                volume_client.volumes.extend(volume.id, parsed_args.size)
+            except Exception as e:
+                LOG.error(_("Failed to set volume size: %s"), e)
+                result += 1
+
+        if parsed_args.no_property:
+            try:
+                volume_client.volumes.delete_metadata(
+                    volume.id, volume.metadata.keys())
+            except Exception as e:
+                LOG.error(_("Failed to clean volume properties: %s"), e)
+                result += 1
 
         if parsed_args.property:
-            volume_client.volumes.set_metadata(volume.id, parsed_args.property)
+            try:
+                volume_client.volumes.set_metadata(
+                    volume.id,
+                    parsed_args.property)
+            except Exception as e:
+                LOG.error(_("Failed to set volume property: %s"), e)
+                result += 1
         if parsed_args.bootable or parsed_args.non_bootable:
             try:
                 volume_client.volumes.set_bootable(
                     volume.id, parsed_args.bootable)
             except Exception as e:
                 LOG.error(_("Failed to set volume bootable property: %s"), e)
+                result += 1
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+                result += 1
         kwargs = {}
         if parsed_args.name:
             kwargs['display_name'] = parsed_args.name
         if parsed_args.description:
             kwargs['display_description'] = parsed_args.description
         if kwargs:
-            volume_client.volumes.update(volume.id, **kwargs)
+            try:
+                volume_client.volumes.update(volume.id, **kwargs)
+            except Exception as e:
+                LOG.error(_("Failed to update volume display name "
+                          "or display description: %s"), e)
+                result += 1
+
+        if result > 0:
+            raise exceptions.CommandError(_("One or more of the "
+                                          "set operations failed"))
 
 
 class ShowVolume(command.ShowOne):
-    """Show volume details"""
+    _description = _("Show volume details")
 
     def get_parser(self, prog_name):
         parser = super(ShowVolume, self).get_parser(prog_name)
@@ -453,7 +580,7 @@ class ShowVolume(command.ShowOne):
 
 
 class UnsetVolume(command.Command):
-    """Unset volume properties"""
+    _description = _("Unset volume properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetVolume, self).get_parser(prog_name)
