@@ -17,6 +17,7 @@
 import copy
 import logging
 
+from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
@@ -29,7 +30,7 @@ LOG = logging.getLogger(__name__)
 
 
 class CreateVolumeBackup(command.ShowOne):
-    """Create new volume backup"""
+    _description = _("Create new volume backup")
 
     def get_parser(self, prog_name):
         parser = super(CreateVolumeBackup, self).get_parser(prog_name)
@@ -94,7 +95,7 @@ class CreateVolumeBackup(command.ShowOne):
 
 
 class CreateBackup(CreateVolumeBackup):
-    """Create new backup"""
+    _description = _("Create new backup")
 
     # TODO(Huanxuan Ao): Remove this class and ``backup create`` command
     #                    two cycles after Newton.
@@ -111,7 +112,7 @@ class CreateBackup(CreateVolumeBackup):
 
 
 class DeleteVolumeBackup(command.Command):
-    """Delete volume backup(s)"""
+    _description = _("Delete volume backup(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteVolumeBackup, self).get_parser(prog_name)
@@ -152,7 +153,7 @@ class DeleteVolumeBackup(command.Command):
 
 
 class DeleteBackup(DeleteVolumeBackup):
-    """Delete backup(s)"""
+    _description = _("Delete backup(s)")
 
     # TODO(Huanxuan Ao): Remove this class and ``backup delete`` command
     #                    two cycles after Newton.
@@ -169,7 +170,7 @@ class DeleteBackup(DeleteVolumeBackup):
 
 
 class ListVolumeBackup(command.Lister):
-    """List volume backups"""
+    _description = _("List volume backups")
 
     def get_parser(self, prog_name):
         parser = super(ListVolumeBackup, self).get_parser(prog_name)
@@ -179,9 +180,48 @@ class ListVolumeBackup(command.Lister):
             default=False,
             help=_("List additional fields in output")
         )
+        parser.add_argument(
+            "--name",
+            metavar="<name>",
+            help=_("Filters results by the backup name")
+        )
+        parser.add_argument(
+            "--status",
+            metavar="<status>",
+            choices=['creating', 'available', 'deleting',
+                     'error', 'restoring', 'error_restoring'],
+            help=_("Filters results by the backup status "
+                   "('creating', 'available', 'deleting', "
+                   "'error', 'restoring' or 'error_restoring')")
+        )
+        parser.add_argument(
+            "--volume",
+            metavar="<volume>",
+            help=_("Filters results by the volume which they "
+                   "backup (name or ID)")
+        )
+        parser.add_argument(
+            '--marker',
+            metavar='<marker>',
+            help=_('The last backup of the previous page (name or ID)'),
+        )
+        parser.add_argument(
+            '--limit',
+            type=int,
+            action=parseractions.NonNegativeAction,
+            metavar='<limit>',
+            help=_('Maximum number of backups to display'),
+        )
+        parser.add_argument(
+            '--all-projects',
+            action='store_true',
+            default=False,
+            help=_('Include all projects (admin only)'),
+        )
         return parser
 
     def take_action(self, parsed_args):
+        volume_client = self.app.client_manager.volume
 
         def _format_volume_id(volume_id):
             """Return a volume name if available
@@ -207,13 +247,31 @@ class ListVolumeBackup(command.Lister):
         # Cache the volume list
         volume_cache = {}
         try:
-            for s in self.app.client_manager.volume.volumes.list():
+            for s in volume_client.volumes.list():
                 volume_cache[s.id] = s
         except Exception:
             # Just forget it if there's any trouble
             pass
 
-        data = self.app.client_manager.volume.backups.list()
+        filter_volume_id = None
+        if parsed_args.volume:
+            filter_volume_id = utils.find_resource(volume_client.volumes,
+                                                   parsed_args.volume).id
+        marker_backup_id = None
+        if parsed_args.marker:
+            marker_backup_id = utils.find_resource(volume_client.backups,
+                                                   parsed_args.marker).id
+        search_opts = {
+            'name': parsed_args.name,
+            'status': parsed_args.status,
+            'volume_id': filter_volume_id,
+            'all_tenants': parsed_args.all_projects,
+        }
+        data = volume_client.backups.list(
+            search_opts=search_opts,
+            marker=marker_backup_id,
+            limit=parsed_args.limit,
+        )
 
         return (column_headers,
                 (utils.get_item_properties(
@@ -223,7 +281,7 @@ class ListVolumeBackup(command.Lister):
 
 
 class ListBackup(ListVolumeBackup):
-    """List backups"""
+    _description = _("List backups")
 
     # TODO(Huanxuan Ao): Remove this class and ``backup list`` command
     #                    two cycles after Newton.
@@ -240,7 +298,7 @@ class ListBackup(ListVolumeBackup):
 
 
 class RestoreVolumeBackup(command.ShowOne):
-    """Restore volume backup"""
+    _description = _("Restore volume backup")
 
     def get_parser(self, prog_name):
         parser = super(RestoreVolumeBackup, self).get_parser(prog_name)
@@ -265,7 +323,7 @@ class RestoreVolumeBackup(command.ShowOne):
 
 
 class RestoreBackup(RestoreVolumeBackup):
-    """Restore backup"""
+    _description = _("Restore backup")
 
     # TODO(Huanxuan Ao): Remove this class and ``backup restore`` command
     #                    two cycles after Newton.
@@ -281,8 +339,70 @@ class RestoreBackup(RestoreVolumeBackup):
         return super(RestoreBackup, self).take_action(parsed_args)
 
 
+class SetVolumeBackup(command.Command):
+    _description = _("Set volume backup properties")
+
+    def get_parser(self, prog_name):
+        parser = super(SetVolumeBackup, self).get_parser(prog_name)
+        parser.add_argument(
+            "backup",
+            metavar="<backup>",
+            help=_("Backup to modify (name or ID)")
+        )
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            help=_('New backup name')
+        )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_('New backup description')
+        )
+        parser.add_argument(
+            '--state',
+            metavar='<state>',
+            choices=['available', 'error'],
+            help=_('New backup state ("available" or "error") (admin only) '
+                   '(This option simply changes the state of the backup '
+                   'in the database with no regard to actual status, '
+                   'exercise caution when using)'),
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        volume_client = self.app.client_manager.volume
+        backup = utils.find_resource(volume_client.backups,
+                                     parsed_args.backup)
+        result = 0
+        if parsed_args.state:
+            try:
+                volume_client.backups.reset_state(
+                    backup.id, parsed_args.state)
+            except Exception as e:
+                LOG.error(_("Failed to set backup state: %s"), e)
+                result += 1
+
+        kwargs = {}
+        if parsed_args.name:
+            kwargs['name'] = parsed_args.name
+        if parsed_args.description:
+            kwargs['description'] = parsed_args.description
+        if kwargs:
+            try:
+                volume_client.backups.update(backup.id, **kwargs)
+            except Exception as e:
+                LOG.error(_("Failed to update backup name "
+                          "or description: %s"), e)
+                result += 1
+
+        if result > 0:
+            raise exceptions.CommandError(_("One or more of the "
+                                          "set operations failed"))
+
+
 class ShowVolumeBackup(command.ShowOne):
-    """Display volume backup details"""
+    _description = _("Display volume backup details")
 
     def get_parser(self, prog_name):
         parser = super(ShowVolumeBackup, self).get_parser(prog_name)
@@ -302,7 +422,7 @@ class ShowVolumeBackup(command.ShowOne):
 
 
 class ShowBackup(ShowVolumeBackup):
-    """Display backup details"""
+    _description = _("Display backup details")
 
     # TODO(Huanxuan Ao): Remove this class and ``backup show`` command
     #                    two cycles after Newton.
